@@ -1,10 +1,12 @@
-// Copyright (C) 2008-2014 Conrad Sanderson
-// Copyright (C) 2008-2014 NICTA (www.nicta.com.au)
-// Copyright (C) 2012 Ryan Curtin
+// Copyright (C) 2008-2016 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Written by Ryan Curtin
 
 
 //! \addtogroup arma_ostream
@@ -55,11 +57,22 @@ arma_ostream::modify_stream(std::ostream& o, const eT* data, const uword n_elem)
   
   bool use_layout_B = false;
   bool use_layout_C = false;
+  bool use_layout_D = false;
   
   for(uword i=0; i<n_elem; ++i)
     {
     const eT val = data[i];
     
+    if(
+      ( cond_rel< (sizeof(eT) > 4) && (is_same_type<uword,eT>::yes || is_same_type<sword,eT>::yes) >::geq(val, eT(+10000000000)) )
+      ||
+      ( cond_rel< (sizeof(eT) > 4) &&  is_same_type<sword,eT>::yes                                 >::leq(val, eT(-10000000000)) )
+      )
+      {
+      use_layout_D = true;
+      break;
+      }
+    
     if(
       ( val >= eT(+100) )
       ||
@@ -96,7 +109,16 @@ arma_ostream::modify_stream(std::ostream& o, const eT* data, const uword n_elem)
       }
     }
   
-  if(use_layout_C == true)
+  if(use_layout_D)
+    {
+    o.setf(ios::scientific);
+    o.setf(ios::right);
+    o.unsetf(ios::fixed);
+    o.precision(4);
+    cell_width = 21;
+    }
+  else
+  if(use_layout_C)
     {
     o.setf(ios::scientific);
     o.setf(ios::right);
@@ -105,7 +127,7 @@ arma_ostream::modify_stream(std::ostream& o, const eT* data, const uword n_elem)
     cell_width = 13;
     }
   else
-  if(use_layout_B == true)
+  if(use_layout_B)
     {
     o.unsetf(ios::scientific);
     o.setf(ios::right);
@@ -196,7 +218,7 @@ arma_ostream::modify_stream(std::ostream& o, typename SpMat<eT>::const_iterator
       }
     }
 
-  if(use_layout_C == true)
+  if(use_layout_C)
     {
     o.setf(ios::scientific);
     o.setf(ios::right);
@@ -205,7 +227,7 @@ arma_ostream::modify_stream(std::ostream& o, typename SpMat<eT>::const_iterator
     cell_width = 13;
     }
   else
-  if(use_layout_B == true)
+  if(use_layout_B)
     {
     o.unsetf(ios::scientific);
     o.setf(ios::right);
@@ -263,13 +285,17 @@ arma_ostream::print_elem_zero(std::ostream& o, const bool modify)
   {
   if(modify == true)
     {
-    const std::streamsize orig_precision = o.precision();
+    const ios::fmtflags   save_flags     = o.flags();
+    const std::streamsize save_precision = o.precision();
     
+    o.unsetf(ios::scientific);
+    o.setf(ios::fixed);
     o.precision(0);
     
     o << eT(0);
     
-    o.precision(orig_precision);
+    o.flags(save_flags);
+    o.precision(save_precision);
     }
   else
     {
@@ -450,9 +476,11 @@ arma_ostream::print(std::ostream& o, const Cube<eT>& x, const bool modify)
     {
     for(uword slice=0; slice < x.n_slices; ++slice)
       {
+      const Mat<eT> tmp(const_cast<eT*>(x.slice_memptr(slice)), x.n_rows, x.n_cols, false);
+      
       o << "[cube slice " << slice << ']' << '\n';
       o.width(cell_width);
-      arma_ostream::print(o, x.slice(slice), false);
+      arma_ostream::print(o, tmp, false);
       o << '\n';
       }
     }
@@ -730,8 +758,9 @@ arma_ostream::print(std::ostream& o, const SpMat<eT>& m, const bool modify)
     const std::streamsize cell_width = modify ? modify_stream<eT>(o, m.begin(), m_n_nonzero) : o.width();
     
     typename SpMat<eT>::const_iterator begin = m.begin();
+    typename SpMat<eT>::const_iterator m_end = m.end();
     
-    while(begin != m.end())
+    while(begin != m_end)
       {
       const uword row = begin.row();
       
