@@ -1,9 +1,11 @@
-// Copyright (C) 2008-2015 Conrad Sanderson
-// Copyright (C) 2008-2015 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2016 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 
@@ -50,10 +52,10 @@
 
 #if defined(ARMA_USE_WRAPPER)
   #define arma_fortran(function) arma_fortran2_prefix(function)
-  #define arma_atlas(function)   wrapper_##function
+  #define arma_wrapper(function) wrapper_##function
 #else
   #define arma_fortran(function) arma_fortran2_noprefix(function)
-  #define arma_atlas(function)   function
+  #define arma_wrapper(function) function
 #endif
 
 #define arma_fortran_prefix(function)   arma_fortran2_prefix(function)
@@ -63,9 +65,33 @@
 #define ARMA_INCFILE_WRAP(x) <x>
 
 
+#if defined(__CYGWIN__)
+  #if defined(ARMA_USE_CXX11)
+    #pragma message ("WARNING: Cygwin may have incomplete support for C++11 features;")
+    #pragma message ("WARNING: if something breaks, you get to keep all the pieces.")
+    #pragma message ("WARNING: to forcefully prevent Armadillo from using C++11 features,")
+    #pragma message ("WARNING: #define ARMA_DONT_USE_CXX11 before #include <armadillo>")
+  #endif
+#endif
+
+
 #if defined(ARMA_USE_CXX11)
+  
   #undef  ARMA_USE_U64S64
   #define ARMA_USE_U64S64
+  
+  #if !defined(ARMA_32BIT_WORD)
+    #undef  ARMA_64BIT_WORD
+    #define ARMA_64BIT_WORD
+  #endif
+  
+  #if defined(ARMA_64BIT_WORD) && defined(SIZE_MAX)
+    #if (SIZE_MAX < 0xFFFFFFFFFFFFFFFFull)
+      #pragma message ("WARNING: disabled use of 64 bit integers, as std::size_t is smaller than 64 bits")
+      #undef ARMA_64BIT_WORD
+    #endif
+  #endif
+  
 #endif
 
 
@@ -75,6 +101,14 @@
 #endif
 
 
+// most compilers can't vectorise slightly elaborate loops;
+// for example clang: http://llvm.org/bugs/show_bug.cgi?id=16358
+#undef  ARMA_SIMPLE_LOOPS
+#define ARMA_SIMPLE_LOOPS
+
+#undef ARMA_GOOD_COMPILER
+
+#undef ARMA_HAVE_TR1
 #undef ARMA_HAVE_GETTIMEOFDAY
 #undef ARMA_HAVE_SNPRINTF
 #undef ARMA_HAVE_ISFINITE
@@ -82,16 +116,9 @@
 #undef ARMA_HAVE_ISINF
 #undef ARMA_HAVE_ISNAN
 
+
 #if (defined(_POSIX_C_SOURCE) && (_POSIX_C_SOURCE >= 200112L))
   #define ARMA_HAVE_GETTIMEOFDAY
-  
-  #if defined(__GNUG__)
-    #define ARMA_HAVE_SNPRINTF
-    #define ARMA_HAVE_ISFINITE
-    #define ARMA_HAVE_LOG1P
-    #define ARMA_HAVE_ISINF
-    #define ARMA_HAVE_ISNAN
-  #endif
 #endif
 
 
@@ -132,7 +159,13 @@
 #endif
 
 
-#if defined(__GNUG__) && !defined(__clang__) && !defined(__INTEL_COMPILER) && !defined(__NVCC__)
+#if (defined(__GNUG__) || defined(__GNUC__)) && (defined(__clang__) || defined(__INTEL_COMPILER) || defined(__NVCC__) || defined(__CUDACC__) || defined(__PGI) || defined(__PATHSCALE__))
+  #undef  ARMA_FAKE_GCC
+  #define ARMA_FAKE_GCC
+#endif
+
+
+#if defined(__GNUG__) && !defined(ARMA_FAKE_GCC)
   
   #undef  ARMA_GCC_VERSION
   #define ARMA_GCC_VERSION (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
@@ -146,7 +179,6 @@
     // due to http://gcc.gnu.org/bugzilla/show_bug.cgi?id=53549
   #endif
   
-  #undef  ARMA_GOOD_COMPILER
   #define ARMA_GOOD_COMPILER
   
   #undef  arma_pure
@@ -175,15 +207,13 @@
   #if defined(ARMA_USE_CXX11)
     #if (ARMA_GCC_VERSION < 40800)
       #pragma message ("WARNING: compiler is in C++11 mode, but it has incomplete support for C++11 features;")
-      #pragma message ("WARNING: if something breaks, you get to keep all the pieces")
-      #pragma message ("WARNING: To forcefully prevent Armadillo from using C++11 features,")
+      #pragma message ("WARNING: if something breaks, you get to keep all the pieces.")
+      #pragma message ("WARNING: to forcefully prevent Armadillo from using C++11 features,")
       #pragma message ("WARNING: #define ARMA_DONT_USE_CXX11 before #include <armadillo>")
       #define ARMA_DONT_USE_CXX11_CHRONO
     #endif
   #endif
   
-  #undef ARMA_HAVE_TR1
-  
   #if !defined(ARMA_USE_CXX11)
     #if defined(_GLIBCXX_USE_C99_MATH_TR1) && defined(_GLIBCXX_USE_C99_COMPLEX_TR1)
       #define ARMA_HAVE_TR1
@@ -202,27 +232,36 @@
     #define ARMA_HAVE_GCC_ASSUME_ALIGNED
   #endif
   
+  // gcc's vectoriser can handle elaborate loops
+  #undef ARMA_SIMPLE_LOOPS
+  
   #if defined(__OPTIMIZE_SIZE__)
-    #undef  ARMA_SIMPLE_LOOPS
     #define ARMA_SIMPLE_LOOPS
   #endif
   
+  #if (defined(_POSIX_C_SOURCE) && (_POSIX_C_SOURCE >= 200112L))
+    #define ARMA_HAVE_SNPRINTF
+    #define ARMA_HAVE_ISFINITE
+    #define ARMA_HAVE_LOG1P
+    #define ARMA_HAVE_ISINF
+    #define ARMA_HAVE_ISNAN
+  #endif
+  
   #undef ARMA_GCC_VERSION
   
 #endif
 
 
-#if defined(__clang__) && !defined(__INTEL_COMPILER)
-  #undef ARMA_HAVE_TR1
+#if defined(__clang__) && (defined(__INTEL_COMPILER) || defined(__NVCC__) || defined(__CUDACC__) || defined(__PGI) || defined(__PATHSCALE__))
+  #undef  ARMA_FAKE_CLANG
+  #define ARMA_FAKE_CLANG
+#endif
+
+
+#if defined(__clang__) && !defined(ARMA_FAKE_CLANG)
   
-  #undef  ARMA_GOOD_COMPILER
   #define ARMA_GOOD_COMPILER
   
-  // clang's vectoriser has trouble dealing with slightly more elaborate loops
-  // http://llvm.org/bugs/show_bug.cgi?id=16358
-  #undef  ARMA_SIMPLE_LOOPS
-  #define ARMA_SIMPLE_LOOPS
-  
   #if !defined(__has_attribute)
     #define __has_attribute(x) 0
   #endif
@@ -290,11 +329,19 @@
   
   #if defined(__apple_build_version__)
     #undef ARMA_USE_EXTERN_CXX11_RNG
-    // because Apple engineers are too lazy to implement thread_local
+    // TODO: check the status of support for "extern thread_local" in clang shipped with Mac OS X
+  #endif
+  
+  #if (defined(_POSIX_C_SOURCE) && (_POSIX_C_SOURCE >= 200112L))
+    #define ARMA_HAVE_SNPRINTF
+    #define ARMA_HAVE_ISFINITE
+    #define ARMA_HAVE_LOG1P
+    #define ARMA_HAVE_ISINF
+    #define ARMA_HAVE_ISNAN
   #endif
   
 #endif
-  
+
 
 #if defined(__INTEL_COMPILER)
   
@@ -302,16 +349,10 @@
     #error "*** Need a newer compiler ***"
   #endif
   
-  #undef ARMA_GOOD_COMPILER
-  #undef ARMA_HAVE_TR1
-  
   #undef  ARMA_HAVE_GCC_ASSUME_ALIGNED
   #undef  ARMA_HAVE_ICC_ASSUME_ALIGNED
   #define ARMA_HAVE_ICC_ASSUME_ALIGNED
   
-  #undef  ARMA_SIMPLE_LOOPS
-  #define ARMA_SIMPLE_LOOPS
-  
 #endif
 
 
@@ -328,29 +369,21 @@
   #endif
   
   #if defined(ARMA_USE_CXX11)
-    #if (_MSC_VER < 1800)
+    #if (_MSC_VER < 1900)
       #pragma message ("WARNING: compiler is in C++11 mode, but it has incomplete support for C++11 features;")
-      #pragma message ("WARNING: if something breaks, you get to keep all the pieces")
+      #pragma message ("WARNING: if something breaks, you get to keep all the pieces.")
+      #pragma message ("WARNING: to forcefully prevent Armadillo from using C++11 features,")
+      #pragma message ("WARNING: #define ARMA_DONT_USE_CXX11 before #include <armadillo>")
     #endif
   #endif
   
-  #undef  ARMA_SIMPLE_LOOPS
-  #define ARMA_SIMPLE_LOOPS
-  
-  #undef ARMA_GOOD_COMPILER
-  #undef ARMA_HAVE_SNPRINTF
-  #undef ARMA_HAVE_ISFINITE
-  #undef ARMA_HAVE_LOG1P
-  #undef ARMA_HAVE_ISINF
-  #undef ARMA_HAVE_ISNAN
-  #undef ARMA_HAVE_TR1
-  
   // #undef  arma_inline
   // #define arma_inline inline __forceinline
   
   #pragma warning(push)
   
   #pragma warning(disable: 4127)  // conditional expression is constant
+  #pragma warning(disable: 4244)  // possible loss of data when converting types
   #pragma warning(disable: 4510)  // default constructor could not be generated
   #pragma warning(disable: 4511)  // copy constructor can't be generated
   #pragma warning(disable: 4512)  // assignment operator can't be generated
@@ -361,6 +394,7 @@
   #pragma warning(disable: 4624)  // destructor can't be generated
   #pragma warning(disable: 4625)  // copy constructor can't be generated
   #pragma warning(disable: 4626)  // assignment operator can't be generated
+  #pragma warning(disable: 4702)  // unreachable code
   #pragma warning(disable: 4710)  // function not inlined
   #pragma warning(disable: 4711)  // call was inlined
   #pragma warning(disable: 4714)  // __forceinline can't be inlined
@@ -398,24 +432,7 @@
   #if (__SUNPRO_CC < 0x5100)
     #error "*** Need a newer compiler ***"
   #endif
-  
-  #undef ARMA_HAVE_SNPRINTF
-  #undef ARMA_HAVE_ISFINITE
-  #undef ARMA_HAVE_LOG1P
-  #undef ARMA_HAVE_ISINF
-  #undef ARMA_HAVE_ISNAN
-  #undef ARMA_HAVE_TR1
-  
-#endif
-
-
-#if defined(__NVCC__)
-  #undef ARMA_HAVE_SNPRINTF
-  #undef ARMA_HAVE_ISFINITE
-  #undef ARMA_HAVE_LOG1P
-  #undef ARMA_HAVE_ISINF
-  #undef ARMA_HAVE_ISNAN
-  #undef ARMA_HAVE_TR1
+    
 #endif
 
 
