@@ -1,9 +1,11 @@
-// Copyright (C) 2008-2013 Conrad Sanderson
-// Copyright (C) 2008-2013 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2015 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 //! \addtogroup fn_max
@@ -198,4 +200,22 @@ max(const SpOp<T1, spop_max>& in, const uword dim)
 
 
 
+inline
+uword
+max(const SizeMat& s)
+  {
+  return (std::max)(s.n_rows, s.n_cols);
+  }
+
+
+
+inline
+uword
+max(const SizeCube& s)
+  {
+  return (std::max)( (std::max)(s.n_rows, s.n_cols), s.n_slices );
+  }
+
+
+
 //! @}
