@@ -1,9 +1,11 @@
-// Copyright (C) 2008-2012 Conrad Sanderson
-// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2015 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 //! \addtogroup fn_min
@@ -113,6 +115,28 @@ min(const T& x)
 
 
 
+//! element-wise minimum
+template<typename T1, typename T2>
+arma_inline
+typename
+enable_if2
+  <
+  ( is_arma_type<T1>::value && is_arma_type<T2>::value && is_same_type<typename T1::elem_type, typename T2::elem_type>::value ),
+  const Glue<T1, T2, glue_min>
+  >::result
+min
+  (
+  const T1& X,
+  const T2& Y
+  )
+  {
+  arma_extra_debug_sigprint();
+  
+  return Glue<T1, T2, glue_min>(X, Y);
+  }
+
+
+
 template<typename T1>
 inline
 arma_warn_unused
@@ -174,4 +198,22 @@ min(const SpOp<T1, spop_min>& in, const uword dim)
 
 
 
+inline
+uword
+min(const SizeMat& s)
+  {
+  return (std::min)(s.n_rows, s.n_cols);
+  }
+
+
+
+inline
+uword
+min(const SizeCube& s)
+  {
+  return (std::min)( (std::min)(s.n_rows, s.n_cols), s.n_slices );
+  }
+
+
+
 //! @}
