@@ -1,9 +1,11 @@
-// Copyright (C) 2008-2013 Conrad Sanderson
-// Copyright (C) 2008-2013 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2015 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 //! \addtogroup fn_randu
@@ -73,6 +75,17 @@ randu(const uword n_rows, const uword n_cols)
 
 
 
+arma_inline
+const Gen<mat, gen_randu>
+randu(const SizeMat& s)
+  {
+  arma_extra_debug_sigprint();
+  
+  return Gen<mat, gen_randu>(s.n_rows, s.n_cols);
+  }
+
+
+
 template<typename obj_type>
 arma_inline
 const Gen<obj_type, gen_randu>
@@ -96,6 +109,19 @@ randu(const uword n_rows, const uword n_cols, const typename arma_Mat_Col_Row_on
 
 
 
+template<typename obj_type>
+arma_inline
+const Gen<obj_type, gen_randu>
+randu(const SizeMat& s, const typename arma_Mat_Col_Row_only<obj_type>::result* junk = 0)
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  return randu<obj_type>(s.n_rows, s.n_cols);
+  }
+
+
+
 arma_inline
 const GenCube<cube::elem_type, gen_randu>
 randu(const uword n_rows, const uword n_cols, const uword n_slices)
@@ -107,6 +133,17 @@ randu(const uword n_rows, const uword n_cols, const uword n_slices)
 
 
 
+arma_inline
+const GenCube<cube::elem_type, gen_randu>
+randu(const SizeCube& s)
+  {
+  arma_extra_debug_sigprint();
+  
+  return GenCube<cube::elem_type, gen_randu>(s.n_rows, s.n_cols, s.n_slices);
+  }
+
+
+
 template<typename cube_type>
 arma_inline
 const GenCube<typename cube_type::elem_type, gen_randu>
@@ -120,4 +157,17 @@ randu(const uword n_rows, const uword n_cols, const uword n_slices, const typena
 
 
 
+template<typename cube_type>
+arma_inline
+const GenCube<typename cube_type::elem_type, gen_randu>
+randu(const SizeCube& s, const typename arma_Cube_only<cube_type>::result* junk = 0)
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  return GenCube<typename cube_type::elem_type, gen_randu>(s.n_rows, s.n_cols, s.n_slices);
+  }
+
+
+
 //! @}
