@@ -1,9 +1,12 @@
-// Copyright (C) 2012 Conrad Sanderson
-// Copyright (C) 2012 Ryan Curtin
+// Copyright (C) 2012-2015 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Written by Ryan Curtin
 
 
 //! \addtogroup fn_speye
@@ -39,6 +42,19 @@ speye(const uword n_rows, const uword n_cols, const typename arma_SpMat_SpCol_Sp
 
 
 
+template<typename obj_type>
+inline
+obj_type
+speye(const SizeMat& s, const typename arma_SpMat_SpCol_SpRow_only<obj_type>::result* junk = NULL)
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  return speye<obj_type>(s.n_rows, s.n_cols);
+  }
+
+
+
 // Convenience shortcut method (no template parameter necessary)
 inline
 sp_mat
@@ -55,4 +71,19 @@ speye(const uword n_rows, const uword n_cols)
 
 
 
+inline
+sp_mat
+speye(const SizeMat& s)
+  {
+  arma_extra_debug_sigprint();
+  
+  sp_mat out;
+  
+  out.eye(s.n_rows, s.n_cols);
+  
+  return out;
+  }
+
+
+
 //! @}
