@@ -1,9 +1,11 @@
-// Copyright (C) 2008-2014 Conrad Sanderson
-// Copyright (C) 2008-2014 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2015 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 //! \addtogroup glue_times
@@ -11,12 +13,12 @@
 
 
 
-template<bool is_eT_blas_type>
+template<bool do_inv_detect>
 template<typename T1, typename T2>
 arma_hot
 inline
 void
-glue_times_redirect2_helper<is_eT_blas_type>::apply(Mat<typename T1::elem_type>& out, const Glue<T1,T2,glue_times>& X)
+glue_times_redirect2_helper<do_inv_detect>::apply(Mat<typename T1::elem_type>& out, const Glue<T1,T2,glue_times>& X)
   {
   arma_extra_debug_sigprint();
   
@@ -73,71 +75,45 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
   
   typedef typename T1::elem_type eT;
   
-  if(strip_inv<T1>::do_inv == false)
+  if(strip_inv<T1>::do_inv == true)
     {
-    const partial_unwrap<T1> tmp1(X.A);
-    const partial_unwrap<T2> tmp2(X.B);
+    // replace inv(A)*B with solve(A,B)
     
-    const typename partial_unwrap<T1>::stored_type& A = tmp1.M;
-    const typename partial_unwrap<T2>::stored_type& B = tmp2.M;
-    
-    const bool use_alpha = partial_unwrap<T1>::do_times || partial_unwrap<T2>::do_times;
-    const eT       alpha = use_alpha ? (tmp1.get_val() * tmp2.get_val()) : eT(0);
-    
-    const bool alias = tmp1.is_alias(out) || tmp2.is_alias(out);
-    
-    if(alias == false)
-      {
-      glue_times::apply
-        <
-        eT,
-        partial_unwrap<T1>::do_trans,
-        partial_unwrap<T2>::do_trans,
-        (partial_unwrap<T1>::do_times || partial_unwrap<T2>::do_times)
-        >
-        (out, A, B, alpha);
-      }
-    else
-      {
-      Mat<eT> tmp;
-      
-      glue_times::apply
-        <
-        eT,
-        partial_unwrap<T1>::do_trans,
-        partial_unwrap<T2>::do_trans,
-        (partial_unwrap<T1>::do_times || partial_unwrap<T2>::do_times)
-        >
-        (tmp, A, B, alpha);
-      
-      out.steal_mem(tmp);
-      }
-    }
-  else
-    {
     arma_extra_debug_print("glue_times_redirect<2>::apply(): detected inv(A)*B");
     
     const strip_inv<T1> A_strip(X.A);
     
     Mat<eT> A = A_strip.M;
     
-    arma_debug_check( (A.is_square() == false), "inv(): given matrix is not square" );
+    arma_debug_check( (A.is_square() == false), "inv(): given matrix must be square sized" );
     
     const unwrap_check<T2> B_tmp(X.B, out);
     const Mat<eT>& B = B_tmp.M;
     
-    glue_solve::solve_direct( out, A, B, A_strip.slow );
+    arma_debug_assert_mul_size(A, B, "matrix multiplication");
+    
+    const bool status = auxlib::solve_square_fast(out, A, B);
+    
+    if(status == false)
+      {
+      out.reset();
+      arma_bad("matrix multiplication: inverse of singular matrix; suggest to use solve() instead");
+      }
+    
+    return;
     }
+  
+  glue_times_redirect2_helper<false>::apply(out, X);
   }
 
 
 
-template<bool is_eT_blas_type>
+template<bool do_inv_detect>
 template<typename T1, typename T2, typename T3>
 arma_hot
 inline
 void
-glue_times_redirect3_helper<is_eT_blas_type>::apply(Mat<typename T1::elem_type>& out, const Glue< Glue<T1,T2,glue_times>, T3, glue_times>& X)
+glue_times_redirect3_helper<do_inv_detect>::apply(Mat<typename T1::elem_type>& out, const Glue< Glue<T1,T2,glue_times>, T3, glue_times>& X)
   {
   arma_extra_debug_sigprint();
   
@@ -199,58 +175,55 @@ glue_times_redirect3_helper<true>::apply(Mat<typename T1::elem_type>& out, const
   {
   arma_extra_debug_sigprint();
   
-  // TODO: investigate detecting inv(A)*B*C and replacing with solve(A,B)*C
-  
   typedef typename T1::elem_type eT;
   
-  if(strip_inv<T2>::do_inv == false)
+  if(strip_inv<T1>::do_inv == true)
     {
-    // we have exactly 3 objects
-    // hence we can safely expand X as X.A.A, X.A.B and X.B
+    // replace inv(A)*B*C with solve(A,B*C);
+    
+    arma_extra_debug_print("glue_times_redirect<3>::apply(): detected inv(A)*B*C");
+    
+    const strip_inv<T1> A_strip(X.A.A);
+    
+    Mat<eT> A = A_strip.M;
+    
+    arma_debug_check( (A.is_square() == false), "inv(): given matrix must be square sized" );
     
-    const partial_unwrap<T1> tmp1(X.A.A);
     const partial_unwrap<T2> tmp2(X.A.B);
     const partial_unwrap<T3> tmp3(X.B  );
     
-    const typename partial_unwrap<T1>::stored_type& A = tmp1.M;
     const typename partial_unwrap<T2>::stored_type& B = tmp2.M;
     const typename partial_unwrap<T3>::stored_type& C = tmp3.M;
     
-    const bool use_alpha = partial_unwrap<T1>::do_times || partial_unwrap<T2>::do_times || partial_unwrap<T3>::do_times;
-    const eT       alpha = use_alpha ? (tmp1.get_val() * tmp2.get_val() * tmp3.get_val()) : eT(0);
+    const bool use_alpha = partial_unwrap<T2>::do_times || partial_unwrap<T3>::do_times;
+    const eT       alpha = use_alpha ? (tmp2.get_val() * tmp3.get_val()) : eT(0);
     
-    const bool alias = tmp1.is_alias(out) || tmp2.is_alias(out) || tmp3.is_alias(out);
+    Mat<eT> BC;
     
-    if(alias == false)
+    glue_times::apply
+      <
+      eT,
+      partial_unwrap<T2>::do_trans,
+      partial_unwrap<T3>::do_trans,
+      (partial_unwrap<T2>::do_times || partial_unwrap<T3>::do_times)
+      >
+      (BC, B, C, alpha);
+    
+    arma_debug_assert_mul_size(A, BC, "matrix multiplication");
+    
+    const bool status = auxlib::solve_square_fast(out, A, BC);
+    
+    if(status == false)
       {
-      glue_times::apply
-        <
-        eT,
-        partial_unwrap<T1>::do_trans,
-        partial_unwrap<T2>::do_trans,
-        partial_unwrap<T3>::do_trans,
-        (partial_unwrap<T1>::do_times || partial_unwrap<T2>::do_times || partial_unwrap<T3>::do_times)
-        >
-        (out, A, B, C, alpha);
-      }
-    else
-      {
-      Mat<eT> tmp;
-      
-      glue_times::apply
-        <
-        eT,
-        partial_unwrap<T1>::do_trans,
-        partial_unwrap<T2>::do_trans,
-        partial_unwrap<T3>::do_trans,
-        (partial_unwrap<T1>::do_times || partial_unwrap<T2>::do_times || partial_unwrap<T3>::do_times)
-        >
-        (tmp, A, B, C, alpha);
-      
-      out.steal_mem(tmp);
+      out.reset();
+      arma_bad("matrix multiplication: inverse of singular matrix; suggest to use solve() instead");
       }
+    
+    return;
     }
-  else
+  
+  
+  if(strip_inv<T2>::do_inv == true)
     {
     // replace A*inv(B)*C with A*solve(B,C)
     
@@ -260,14 +233,23 @@ glue_times_redirect3_helper<true>::apply(Mat<typename T1::elem_type>& out, const
     
     Mat<eT> B = B_strip.M;
     
-    arma_debug_check( (B.is_square() == false), "inv(): given matrix is not square" );
+    arma_debug_check( (B.is_square() == false), "inv(): given matrix must be square sized" );
     
     const unwrap<T3> C_tmp(X.B);
     const Mat<eT>& C = C_tmp.M;
     
+    arma_debug_assert_mul_size(B, C, "matrix multiplication");
+    
     Mat<eT> solve_result;
     
-    glue_solve::solve_direct( solve_result, B, C, B_strip.slow );
+    const bool status = auxlib::solve_square_fast(solve_result, B, C);
+    
+    if(status == false)
+      {
+      out.reset();
+      arma_bad("matrix multiplication: inverse of singular matrix; suggest to use solve() instead");
+      return;
+      }
     
     const partial_unwrap_check<T1> tmp1(X.A.A, out);
     
@@ -284,7 +266,12 @@ glue_times_redirect3_helper<true>::apply(Mat<typename T1::elem_type>& out, const
       partial_unwrap_check<T1>::do_times
       >
       (out, A, solve_result, alpha);
+    
+    return;
     }
+  
+  
+  glue_times_redirect3_helper<false>::apply(out, X);
   }
 
 
@@ -443,7 +430,7 @@ glue_times::apply(Mat<typename T1::elem_type>& out, const Glue<T1,T2,glue_times>
   
   const sword N_mat = 1 + depth_lhs< glue_times, Glue<T1,T2,glue_times> >::num;
   
-  arma_extra_debug_print(arma_boost::format("N_mat = %d") % N_mat);
+  arma_extra_debug_print(arma_str::format("N_mat = %d") % N_mat);
   
   glue_times_redirect<N_mat>::apply(out, X);
   }
@@ -458,42 +445,7 @@ glue_times::apply_inplace(Mat<typename T1::elem_type>& out, const T1& X)
   {
   arma_extra_debug_sigprint();
   
-  typedef typename T1::elem_type eT;
-  
-  const unwrap_check<T1> B_tmp(X, out);
-  const Mat<eT>& B     = B_tmp.M;
-  
-  arma_debug_assert_mul_size(out, B, "matrix multiplication");
-  
-  const uword out_n_rows = out.n_rows;
-  const uword out_n_cols = out.n_cols;
-  
-  if(out_n_cols == B.n_cols)
-    {
-    // size of resulting matrix is the same as 'out'
-    
-    podarray<eT> tmp(out_n_cols);
-    
-    eT* tmp_rowdata = tmp.memptr();
-    
-    for(uword row=0; row < out_n_rows; ++row)
-      {
-      tmp.copy_row(out, row);
-      
-      for(uword col=0; col < out_n_cols; ++col)
-        {
-        out.at(row,col) = op_dot::direct_dot( out_n_cols, tmp_rowdata, B.colptr(col) );
-        }
-      }
-    
-    }
-  else
-    {
-    const Mat<eT> tmp(out);
-    
-    glue_times::apply<eT, false, false, false>(out, tmp, B, eT(1));
-    }
-  
+  out = out * X;
   }
 
 
@@ -509,6 +461,17 @@ glue_times::apply_inplace_plus(Mat<typename T1::elem_type>& out, const Glue<T1,
   typedef typename T1::elem_type            eT;
   typedef typename get_pod_type<eT>::result  T;
   
+  if( (is_outer_product<T1>::value) || (has_op_inv<T1>::value) || (has_op_inv<T2>::value) )
+    {
+    // partial workaround for corner cases
+    
+    const Mat<eT> tmp(X);
+    
+    if(sign > sword(0))  { out += tmp; }  else  { out -= tmp; }
+    
+    return;
+    }
+  
   const partial_unwrap_check<T1> tmp1(X.A, out);
   const partial_unwrap_check<T2> tmp2(X.B, out);
   
@@ -841,38 +804,30 @@ glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue
   
   if( (strip_diagmat<T1>::do_diagmat == true) && (strip_diagmat<T2>::do_diagmat == false) )
     {
+    arma_extra_debug_print("glue_times_diag::apply(): diagmat(A) * B");
+    
     const diagmat_proxy_check<T1_stripped> A(S1.M, out);
     
     const unwrap_check<T2> tmp(X.B, out);
     const Mat<eT>& B     = tmp.M;
     
-    const uword A_n_elem = A.n_elem;
+    const uword A_n_rows = A.n_rows;
+    const uword A_n_cols = A.n_cols;
+    const uword A_length = (std::min)(A_n_rows, A_n_cols);
+    
     const uword B_n_rows = B.n_rows;
     const uword B_n_cols = B.n_cols;
     
-    arma_debug_assert_mul_size(A_n_elem, A_n_elem, B_n_rows, B_n_cols, "matrix multiplication");
+    arma_debug_assert_mul_size(A_n_rows, A_n_cols, B_n_rows, B_n_cols, "matrix multiplication");
     
-    out.set_size(A_n_elem, B_n_cols);
+    out.zeros(A_n_rows, B_n_cols);
     
     for(uword col=0; col < B_n_cols; ++col)
       {
             eT* out_coldata = out.colptr(col);
-      const eT* B_coldata   = B.colptr(col);
+      const eT*   B_coldata =   B.colptr(col);
       
-      uword i,j;
-      for(i=0, j=1; j < B_n_rows; i+=2, j+=2)
-        {
-        eT tmp_i = A[i];
-        eT tmp_j = A[j];
-        
-        tmp_i *= B_coldata[i];
-        tmp_j *= B_coldata[j];
-        
-        out_coldata[i] = tmp_i;
-        out_coldata[j] = tmp_j;
-        }
-      
-      if(i < B_n_rows)
+      for(uword i=0; i < A_length; ++i)
         {
         out_coldata[i] = A[i] * B_coldata[i];
         }
@@ -881,6 +836,8 @@ glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue
   else
   if( (strip_diagmat<T1>::do_diagmat == false) && (strip_diagmat<T2>::do_diagmat == true) )
     {
+    arma_extra_debug_print("glue_times_diag::apply(): A * diagmat(B)");
+    
     const unwrap_check<T1> tmp(X.A, out);
     const Mat<eT>& A     = tmp.M;
     
@@ -888,30 +845,23 @@ glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue
     
     const uword A_n_rows = A.n_rows;
     const uword A_n_cols = A.n_cols;
-    const uword B_n_elem = B.n_elem;
     
-    arma_debug_assert_mul_size(A_n_rows, A_n_cols, B_n_elem, B_n_elem, "matrix multiplication");
+    const uword B_n_rows = B.n_rows;
+    const uword B_n_cols = B.n_cols;
+    const uword B_length = (std::min)(B_n_rows, B_n_cols);
     
-    out.set_size(A_n_rows, B_n_elem);
+    arma_debug_assert_mul_size(A_n_rows, A_n_cols, B_n_rows, B_n_cols, "matrix multiplication");
     
-    for(uword col=0; col < A_n_cols; ++col)
+    out.zeros(A_n_rows, B_n_cols);
+    
+    for(uword col=0; col < B_length; ++col)
       {
       const eT  val = B[col];
       
             eT* out_coldata = out.colptr(col);
       const eT*   A_coldata =   A.colptr(col);
       
-      uword i,j;
-      for(i=0, j=1; j < A_n_rows; i+=2, j+=2)
-        {
-        const eT tmp_i = A_coldata[i] * val;
-        const eT tmp_j = A_coldata[j] * val;
-        
-        out_coldata[i] = tmp_i;
-        out_coldata[j] = tmp_j;
-        }
-      
-      if(i < A_n_rows)
+      for(uword i=0; i < A_n_rows; ++i)
         {
         out_coldata[i] = A_coldata[i] * val;
         }
@@ -920,17 +870,21 @@ glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue
   else
   if( (strip_diagmat<T1>::do_diagmat == true) && (strip_diagmat<T2>::do_diagmat == true) )
     {
+    arma_extra_debug_print("glue_times_diag::apply(): diagmat(A) * diagmat(B)");
+    
     const diagmat_proxy_check<T1_stripped> A(S1.M, out);
     const diagmat_proxy_check<T2_stripped> B(S2.M, out);
     
-    const uword A_n_elem = A.n_elem;
-    const uword B_n_elem = B.n_elem;
+    arma_debug_assert_mul_size(A.n_rows, A.n_cols, B.n_rows, B.n_cols, "matrix multiplication");
     
-    arma_debug_assert_mul_size(A_n_elem, A_n_elem, B_n_elem, B_n_elem, "matrix multiplication");
+    out.zeros(A.n_rows, B.n_cols);
     
-    out.zeros(A_n_elem, A_n_elem);
+    const uword A_length = (std::min)(A.n_rows, A.n_cols);
+    const uword B_length = (std::min)(B.n_rows, B.n_cols);
     
-    for(uword i=0; i < A_n_elem; ++i)
+    const uword N = (std::min)(A_length, B_length);
+    
+    for(uword i=0; i < N; ++i)
       {
       out.at(i,i) = A[i] * B[i];
       }
