@@ -1,9 +1,11 @@
-// Copyright (C) 2008-2015 Conrad Sanderson
-// Copyright (C) 2008-2015 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2015 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 //! \addtogroup op_max
@@ -144,7 +146,6 @@ op_max::apply_noalias(Mat<eT>& out, const Mat<eT>& X, const uword dim, const typ
 
 
 template<typename eT>
-arma_pure
 inline
 eT
 op_max::direct_max(const eT* const X, const uword n_elem)
