@@ -1,9 +1,11 @@
-// Copyright (C) 2009-2013 Conrad Sanderson
-// Copyright (C) 2009-2013 NICTA (www.nicta.com.au)
+// Copyright (C) 2009-2013 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 //! \addtogroup running_stat_vec
@@ -81,7 +83,7 @@ running_stat_vec<obj_type>::operator() (const Base<typename running_stat_vec<obj
   {
   arma_extra_debug_sigprint();
   
-  const unwrap<T1>       tmp(X.get_ref());
+  const quasi_unwrap<T1> tmp(X.get_ref());
   const Mat<T>& sample = tmp.M;
   
   if( sample.is_empty() )
@@ -91,7 +93,7 @@ running_stat_vec<obj_type>::operator() (const Base<typename running_stat_vec<obj
   
   if( sample.is_finite() == false )
     {
-    arma_warn(true, "running_stat_vec: sample ignored as it has non-finite elements");
+    arma_debug_warn("running_stat_vec: sample ignored as it has non-finite elements");
     return;
     }
   
@@ -109,7 +111,7 @@ running_stat_vec<obj_type>::operator() (const Base< std::complex<typename runnin
   {
   arma_extra_debug_sigprint();
   
-  const unwrap<T1> tmp(X.get_ref());
+  const quasi_unwrap<T1> tmp(X.get_ref());
   
   const Mat< std::complex<T> >& sample = tmp.M;
   
@@ -120,7 +122,7 @@ running_stat_vec<obj_type>::operator() (const Base< std::complex<typename runnin
   
   if( sample.is_finite() == false )
     {
-    arma_warn(true, "running_stat_vec: sample ignored as it has non-finite elements");
+    arma_debug_warn("running_stat_vec: sample ignored as it has non-finite elements");
     return;
     }
   
