@@ -1,8 +1,11 @@
-// Copyright (C) 2012 Conrad Sanderson
+// Copyright (C) 2012 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// -------------------------------------------------------------------
+// 
+// Written by Conrad Sanderson - http://conradsanderson.id.au
 
 
 //! \addtogroup spop_sum
@@ -15,9 +18,6 @@ class spop_sum
   
   template<typename T1>
   arma_hot inline static void apply(SpMat<typename T1::elem_type>& out, const SpOp<T1, spop_sum>& in);
-  
-  template<typename T1>
-  arma_hot inline static void apply_noalias(SpMat<typename T1::elem_type>& out, const SpProxy<T1>& p, const uword dim);
   };
 
 
