@@ -1,4 +1,4 @@
-// Copyright (C) 2008-2015 National ICT Australia (NICTA)
+// Copyright (C) 2008-2016 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -998,6 +998,63 @@ subview_cube<eT>::operator/= (const Base<eT,T1>& in)
 
 
 
+//! apply a functor to each element
+template<typename eT>
+template<typename functor>
+inline
+void
+subview_cube<eT>::for_each(functor F)
+  {
+  arma_extra_debug_sigprint();
+  
+  Cube<eT>& Q = const_cast< Cube<eT>& >(m);
+  
+  const uword start_col   = aux_col1;
+  const uword start_row   = aux_row1;
+  const uword start_slice = aux_slice1;
+  
+  const uword end_col_plus1   = start_col   + n_cols;
+  const uword end_row_plus1   = start_row   + n_rows;
+  const uword end_slice_plus1 = start_slice + n_slices;
+  
+  for(uword uslice = start_slice; uslice < end_slice_plus1; ++uslice)
+  for(uword ucol   = start_col;     ucol < end_col_plus1;   ++ucol  )
+  for(uword urow   = start_row;     urow < end_row_plus1;   ++urow  )
+    {
+    F( Q.at(urow, ucol, uslice) );
+    }
+  }
+
+
+
+template<typename eT>
+template<typename functor>
+inline
+void
+subview_cube<eT>::for_each(functor F) const
+  {
+  arma_extra_debug_sigprint();
+  
+  const Cube<eT>& Q = m;
+  
+  const uword start_col   = aux_col1;
+  const uword start_row   = aux_row1;
+  const uword start_slice = aux_slice1;
+  
+  const uword end_col_plus1   = start_col   + n_cols;
+  const uword end_row_plus1   = start_row   + n_rows;
+  const uword end_slice_plus1 = start_slice + n_slices;
+  
+  for(uword uslice = start_slice; uslice < end_slice_plus1; ++uslice)
+  for(uword ucol   = start_col;     ucol < end_col_plus1;   ++ucol  )
+  for(uword urow   = start_row;     urow < end_row_plus1;   ++urow  )
+    {
+    F( Q.at(urow, ucol, uslice) );
+    }
+  }
+
+
+
 //! transform each element in the subview using a functor
 template<typename eT>
 template<typename functor>
@@ -1056,6 +1113,62 @@ subview_cube<eT>::imbue(functor F)
 
 
 
+#if defined(ARMA_USE_CXX11)
+  
+  //! apply a lambda function to each slice, where each slice is interpreted as a matrix
+  template<typename eT>
+  inline
+  void
+  subview_cube<eT>::each_slice(const std::function< void(Mat<eT>&) >& F)
+    {
+    arma_extra_debug_sigprint();
+    
+    Mat<eT> tmp1(n_rows, n_cols);
+    Mat<eT> tmp2('j', tmp1.memptr(), n_rows, n_cols);
+    
+    for(uword slice_id=0; slice_id < n_slices; ++slice_id)
+      {
+      for(uword col_id=0; col_id < n_cols; ++col_id)
+        {
+        arrayops::copy( tmp1.colptr(col_id), slice_colptr(slice_id, col_id), n_rows );
+        }
+      
+      F(tmp2);
+      
+      for(uword col_id=0; col_id < n_cols; ++col_id)
+        {
+        arrayops::copy( slice_colptr(slice_id, col_id), tmp1.colptr(col_id), n_rows );
+        }
+      }
+    }
+  
+  
+  
+  template<typename eT>
+  inline
+  void
+  subview_cube<eT>::each_slice(const std::function< void(const Mat<eT>&) >& F) const
+    {
+    arma_extra_debug_sigprint();
+    
+          Mat<eT> tmp1(n_rows, n_cols);
+    const Mat<eT> tmp2('j', tmp1.memptr(), n_rows, n_cols);
+    
+    for(uword slice_id=0; slice_id < n_slices; ++slice_id)
+      {
+      for(uword col_id=0; col_id < n_cols; ++col_id)
+        {
+        arrayops::copy( tmp1.colptr(col_id), slice_colptr(slice_id, col_id), n_rows );
+        }
+      
+      F(tmp2);
+      }
+    }
+  
+#endif
+
+
+
 template<typename eT>
 inline
 void
@@ -1348,7 +1461,7 @@ inline
 eT&
 subview_cube<eT>::operator()(const uword i)
   {
-  arma_debug_check( (i >= n_elem), "subview_cube::operator(): index out of bounds");
+  arma_debug_check( (i >= n_elem), "subview_cube::operator(): index out of bounds" );
   
   const uword in_slice = i / n_elem_slice;
   const uword offset   = in_slice * n_elem_slice;
@@ -1369,7 +1482,7 @@ inline
 eT
 subview_cube<eT>::operator()(const uword i) const
   {
-  arma_debug_check( (i >= n_elem), "subview_cube::operator(): index out of bounds");
+  arma_debug_check( (i >= n_elem), "subview_cube::operator(): index out of bounds" );
   
   const uword in_slice = i / n_elem_slice;
   const uword offset   = in_slice * n_elem_slice;
@@ -1390,7 +1503,7 @@ arma_inline
 eT&
 subview_cube<eT>::operator()(const uword in_row, const uword in_col, const uword in_slice)
   {
-  arma_debug_check( ( (in_row >= n_rows) || (in_col >= n_cols) || (in_slice >= n_slices) ), "subview_cube::operator(): location out of bounds");
+  arma_debug_check( ( (in_row >= n_rows) || (in_col >= n_cols) || (in_slice >= n_slices) ), "subview_cube::operator(): location out of bounds" );
   
   const uword index = (in_slice + aux_slice1)*m.n_elem_slice + (in_col + aux_col1)*m.n_rows + aux_row1 + in_row;
   
@@ -1404,7 +1517,7 @@ arma_inline
 eT
 subview_cube<eT>::operator()(const uword in_row, const uword in_col, const uword in_slice) const
   {
-  arma_debug_check( ( (in_row >= n_rows) || (in_col >= n_cols) || (in_slice >= n_slices) ), "subview_cube::operator(): location out of bounds");
+  arma_debug_check( ( (in_row >= n_rows) || (in_col >= n_cols) || (in_slice >= n_slices) ), "subview_cube::operator(): location out of bounds" );
   
   const uword index = (in_slice + aux_slice1)*m.n_elem_slice + (in_col + aux_col1)*m.n_rows + aux_row1 + in_row;
   
@@ -1547,7 +1660,7 @@ subview_cube<eT>::extract(Cube<eT>& out, const subview_cube<eT>& in)
   const uword n_cols   = in.n_cols;
   const uword n_slices = in.n_slices;
   
-  arma_extra_debug_print(arma_boost::format("out.n_rows = %d   out.n_cols = %d    out.n_slices = %d    in.m.n_rows = %d   in.m.n_cols = %d   in.m.n_slices = %d") % out.n_rows % out.n_cols % out.n_slices % in.m.n_rows % in.m.n_cols % in.m.n_slices);
+  arma_extra_debug_print(arma_str::format("out.n_rows = %d   out.n_cols = %d    out.n_slices = %d    in.m.n_rows = %d   in.m.n_cols = %d   in.m.n_slices = %d") % out.n_rows % out.n_cols % out.n_slices % in.m.n_rows % in.m.n_cols % in.m.n_slices);
   
   
   for(uword slice = 0; slice < n_slices; ++slice)
@@ -1775,6 +1888,19 @@ subview_cube<eT>::plus_inplace(Mat<eT>& out, const subview_cube<eT>& in)
   
   if(in_n_slices == 1)
     {
+    if( (arma_config::debug) && ((out_n_rows != in_n_rows) || (out_n_cols != in_n_cols)) )
+      {
+      std::stringstream tmp;
+      
+      tmp
+        << "in-place addition: "
+        << out_n_rows << 'x' << out_n_cols << " output matrix is incompatible with "
+        <<  in_n_rows << 'x' <<  in_n_cols << 'x' << in_n_slices << " cube interpreted as "
+        <<  in_n_rows << 'x' <<  in_n_cols << " matrix";
+      
+      arma_stop(tmp.str());
+      }
+    
     for(uword col=0; col < in_n_cols; ++col)
       {
       arrayops::inplace_plus( out.colptr(col), in.slice_colptr(0, col), in_n_rows );
@@ -1863,6 +1989,19 @@ subview_cube<eT>::minus_inplace(Mat<eT>& out, const subview_cube<eT>& in)
   
   if(in_n_slices == 1)
     {
+    if( (arma_config::debug) && ((out_n_rows != in_n_rows) || (out_n_cols != in_n_cols)) )
+      {
+      std::stringstream tmp;
+      
+      tmp
+        << "in-place subtraction: "
+        << out_n_rows << 'x' << out_n_cols << " output matrix is incompatible with "
+        <<  in_n_rows << 'x' <<  in_n_cols << 'x' << in_n_slices << " cube interpreted as "
+        <<  in_n_rows << 'x' <<  in_n_cols << " matrix";
+      
+      arma_stop(tmp.str());
+      }
+    
     for(uword col=0; col < in_n_cols; ++col)
       {
       arrayops::inplace_minus( out.colptr(col), in.slice_colptr(0, col), in_n_rows );
@@ -1951,6 +2090,19 @@ subview_cube<eT>::schur_inplace(Mat<eT>& out, const subview_cube<eT>& in)
   
   if(in_n_slices == 1)
     {
+    if( (arma_config::debug) && ((out_n_rows != in_n_rows) || (out_n_cols != in_n_cols)) )
+      {
+      std::stringstream tmp;
+      
+      tmp
+        << "in-place element-wise multiplication: "
+        << out_n_rows << 'x' << out_n_cols << " output matrix is incompatible with "
+        <<  in_n_rows << 'x' <<  in_n_cols << 'x' << in_n_slices << " cube interpreted as "
+        <<  in_n_rows << 'x' <<  in_n_cols << " matrix";
+      
+      arma_stop(tmp.str());
+      }
+    
     for(uword col=0; col < in_n_cols; ++col)
       {
       arrayops::inplace_mul( out.colptr(col), in.slice_colptr(0, col), in_n_rows );
@@ -2039,6 +2191,19 @@ subview_cube<eT>::div_inplace(Mat<eT>& out, const subview_cube<eT>& in)
   
   if(in_n_slices == 1)
     {
+    if( (arma_config::debug) && ((out_n_rows != in_n_rows) || (out_n_cols != in_n_cols)) )
+      {
+      std::stringstream tmp;
+      
+      tmp
+        << "in-place element-wise division: "
+        << out_n_rows << 'x' << out_n_cols << " output matrix is incompatible with "
+        <<  in_n_rows << 'x' <<  in_n_cols << 'x' << in_n_slices << " cube interpreted as "
+        <<  in_n_rows << 'x' <<  in_n_cols << " matrix";
+      
+      arma_stop(tmp.str());
+      }
+    
     for(uword col=0; col < in_n_cols; ++col)
       {
       arrayops::inplace_div( out.colptr(col), in.slice_colptr(0, col), in_n_rows );
