@@ -1,4 +1,4 @@
-// Copyright (C) 2008-2013 National ICT Australia (NICTA)
+// Copyright (C) 2008-2015 National ICT Australia (NICTA)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -34,7 +34,7 @@ class wall_clock
     struct timeval posix_time1;
     struct timeval posix_time2;
   #else
-    clock_t time1;
+    std::clock_t time1;
   #endif
   };
 
